'' =================================================================================================
''
''   File....... jm_parser_demo.spin
''   Purpose.... Command parser demo for PAB
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 30 AUG 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
                                                                 
  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                  
                                                                  
con { pab io pins }                                               
                                                                  
  RX1     = 31                                                  ' programming / terminal
  TX1     = 30                                                   
                                                                 
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28                                                   
                                                                 
  AUD_RT  = 27                                                  ' audio (d/a) outputs
  AUD_LF  = 26                                                   
                                                                 
  LED_27  = 27                                                  ' leds (on audio outputs)
  LED_26  = 26                                                   
                                                                 
  SD_CS   = 25                                                  ' usd card
  SD_DI   = 24                                                   
  SD_CLK  = 23                                                   
  SD_DO   = 22                                                   
                                                                 
  ADC_CS  = 21                                                  ' adc input
  ADC_CLK = 20                                                   
  ADC_DO  = 19                                                   
  ADC_DI  = 18                                                   
                                                                 
  SVO_6   = 17                                                  ' servo headers
  SVO_5   = 16                                                   
  SVO_4   = 15                                                   
  SVO_3   = 14                                                   
  SVO_2   = 13                                                   
  SVO_1   = 12                                                   
                                                                 
  ' app io                                                       
                                                                 
  SPKR    = 15                                                  ' to speaker or peizo through 10uF cap                                         
                                                                 
                                                                 
obj                                                              
                                                                 
' main                                                          ' * master Spin cog
  time       : "jm_time_80"                                     '   timing and delays
  prng       : "jm_prng"                                        '   pseudo-random number generator
  parser     : "jm_parser"                                      '   parsing engine
  io         : "jm_io_basic"                                    '   essential io
  adc        : "jm_adc124s021"                                  '   for onboard adc (foreground)
  term       : "jm_fullduplexserial"                            ' * serial for terminal
' background                                                    ' * support processes
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main | c                                              
                                                                 
  setup

  repeat
    check_command
    {
      other code
    }


pub check_command | c

  c := term.rxcheck                                             ' look for command input
  if (c => 0)                                                    
    if (parser.enqueue(c))                                       
      process_cmd                                                             
      parser.reset                                               
                                                                 
                                                                 
pub setup                                                        
                                                                 
  time.start                                                    ' setup for timing/delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $DEAD_BEEF, cnt <- 2, cnt)         ' seed randomizer                             
                                                                 
  parser.start(@CHAR_SET, @TOKEN_LIST, TOKEN_COUNT, false)      ' setup parsing engine
                                                                 
  io.start(0, 0)                                                ' clear pins (this cog)
                                                                 
' adc.start(ADC_CS, ADC_CLK, ADC_DI, ADC_DO)                    ' connect to adc 
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal *
                                                                 
  bcog := cognew(background, @bstack) + 1                       ' launch background cog *
                                                                 

con

  { ------------------------- }
  {  P A R S E R   S E T U P  }
  { ------------------------- }


dat { valid characters and tokens }

  CHAR_SET      byte    "-_0123456789"                          ' valid characters for this token set
                byte    "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0
  
  TOKEN_LIST    byte    "HIGH", 0                               ' HIGH pin_number 
                byte    "LOW", 0                                ' LOW pin_number   
                byte    "TOGGLE", 0                             ' TOGGLE pin_number 
                byte    "INPUT", 0                              ' INPUT pin_number (0..27) 
                byte    "ANALOG", 0                             ' ANALOG channel (0..3)  
                byte    "TONE", 0                               ' TONE hertz ms
                byte    "BEEP", 0                               ' BEEP or BEEP ms
                byte    "NOW", 0                                ' NOW (shows current millis)


con { enumerated token indexes }

  #00, T_HIGH, T_LOW, T_TOGGLE, T_INPUT                         ' pin io
  #04, T_ANALOG                                                 ' analog in
  #05, T_TONE, T_BEEP                                           ' sound
  #07, T_NOW                                                    ' time
                                                                 
  TOKEN_COUNT = T_NOW+1                                         ' total # of known tokens
                                                                 
                                                                 
pub process_cmd | tidx                                           
                                                                 
  tidx := parser.get_token_id(parser.token_addr(0))             ' get index of command token
                                                                 
  case tidx
    T_HIGH   : set_high
    T_LOW    : set_low
    T_TOGGLE : toggle_pin
    T_INPUT  : read_pin
    T_ANALOG : read_analog
    T_TONE   : play_tone
    T_BEEP   : beeper
    T_NOW    : show_time
    other    : invalid_cmd


pub set_high | ok, pin

  ok := (parser.token_count == 2)                               ' command requires 2 tokens
                                                                 
  if (ok)                                                        
    if (parser.token_is_number(1))                              ' is channel a #?
      pin := parser.token_value(1)                              ' extract it
      ok := ((pin => 0) and (pin =< 27))                        ' validate channel?
    else                                                         
      ok := false                                                
                                                                 
  ifnot (ok)                                                     
    term.str(string("Use: HIGH [0..27]", 13))                    
    return                                                       
                                                                 
  io.high(pin)                                                   
                                                                 
                                                                 
pub set_low | ok, pin                                            
                                                                 
  ok := (parser.token_count == 2)                               ' command requires 2 tokens
                                                                 
  if (ok)                                                        
    if (parser.token_is_number(1))                              ' is channel a #?
      pin := parser.token_value(1)                              ' extract it
      ok := ((pin => 0) and (pin =< 27))                        ' validate channel?
    else                                                         
      ok := false                                                
                                                                 
  ifnot (ok)                                                     
    term.str(string("Use: LOW [0..27]", 13))                     
    return                                                       
                                                                 
  io.low(pin)                                                    
                                                                 
                                                                 
pub toggle_pin  | ok, pin                                        
                                                                 
  ok := (parser.token_count == 2)                               ' command requires 2 tokens
                                                                 
  if (ok)                                                        
    if (parser.token_is_number(1))                              ' is channel a #?
      pin := parser.token_value(1)                              ' extract it
      ok := ((pin => 0) and (pin =< 27))                        ' validate channel?
    else                                                         
      ok := false                                                
                                                                 
  ifnot (ok)                                                     
    term.str(string("Use: TOGGLE [0..27]", 13))                  
    return                                                       

  io.toggle(pin)

             
pub read_pin | ok, pin

  ok := (parser.token_count == 2)                               ' command requires 2 tokens

  if (ok)
    if (parser.token_is_number(1))                              ' is channel a #?
      pin := parser.token_value(1)                              ' extract it
      ok := ((pin => 0) and (pin =< 27))                        ' validate channel?
    else
      ok := false

  ifnot (ok)
    term.str(string("Use: INPUT [0..27]", 13))
    return

  term.tx("0" + io.input(pin))                                  
  term.tx(13)

               
pub read_analog | ok, ch

  ok := (parser.token_count == 2)                               ' command requires 2 tokens

  if (ok)
    if (parser.token_is_number(1))                              ' is channel a #?
      ch := parser.token_value(1)                               ' extract it
      ok := ((ch => 0) and (ch =< 3))                           ' validate channel?
    else
      ok := false

  ifnot (ok)
    term.str(string("Use: ANALOG [0..3]", 13))
    return

  term.dec(read_adc(ch))                                  
  term.tx(13)

             
pub play_tone | ok, hz, ms

  ok := (parser.token_count == 3)                               ' command requires 3 tokens

  if (ok)
    if (parser.token_is_number(1))                              ' is hz a #?
      hz := parser.token_value(1)                               ' extract it
      if (parser.token_is_number(2))                            ' is ms a #?
        ms := parser.token_value(2)                             ' extract it
      else
        ok := false
    else
      ok := false

  ifnot (ok)
    term.str(string("Use: TONE hertz ms", 13))
    return

  tone_on(SPKR, hz, ms)


pub beeper | n, ok, ms

  n := parser.token_count

  ok := ((n == 1) or (n == 2))                                  ' duration is optional

  if (ok)
    if (n == 1)                                                 ' no duration?
      ms := 100                                                 ' set to 0.1s
    else
      if (parser.token_is_number(1))
        ms := parser.token_value(1)
      else
        ok := false  

  ifnot (ok)
    term.str(string("Use: BEEP {ms}", 13))
    return

  tone_on(SPKR, 1000, ms)


pub show_time

  if (parser.token_count == 1)
    term.dec(millis)
    term.tx(13)
  else
    term.str(string("Use: NOW", 13))       

              
pub invalid_cmd

  term.str(string("Invalid command", 13))             
  
                                                                 
con                                                              
                                                                 
  { --------------------- }                                          
  {  B A C K G R O U N D  }                                          
  { --------------------- }


pub tone_on(pin, hz, ms)

'' Play tone on pin
'' -- frequency in hz
'' -- ms is duration in milliseconds

  tone_off                                                      ' kill current tone
  waitcnt(cnt + MS_001)                                         ' ensure it's off
  longmove(@tonepin, @pin, 3)                                   ' copy parameters


pub tone_off

'' Kill tone now

  tonems := 0


pub read_adc(ch)

  if (ch => 0) and (ch =< 3)
    return analog[ch]
  else
    return -1
                                                                 

var

  long  bcog                                                    ' cog running background code
  long  bstack[32]                                              ' stack space for background cog

  long  millis                                                  ' running milliseconds


pri background | t                                              ' launch with cognew()

  io.high(ADC_CS)                                               ' setup ADC pins
  io.low(ADC_CLK)
  io.low(ADC_DI)
  io.input(ADC_DO)
  
  t := cnt                                                      ' sync loop timing
  repeat
    waitcnt(t += MS_001)                                        ' wait 1ms
    ++millis                                                    ' increment millis
    tone_generator
    adc_process


var

  long  tonepin
  long  tonefreq
  long  tonems


pri tone_generator                                              ' call from background()  

  if (tonems > 0)                                               ' if timer running
    if (ctra == 0)                                              ' tone not enabled yet?
      frqa := ($8000_0000 / (CLK_FREQ / tonefreq)) << 1         ' setup for frequency
      ctra := ((%00100) << 26) | tonepin                        ' activate tone on pin
      dira[tonepin] := 1                                        ' make pin an output
    else
      --tonems
  else
    ctra := 0                                                   ' stop tone
    outa[tonepin] := 0                                          ' clear pin


var

  long  analog[4]                                               ' results for foreground use

  long  astate
  long  ach
  long  actrl
  long  awork


pri adc_process                                                 ' call from background()  

  case astate
    0 :
      outa[ADC_CS] := 0                                         ' activate adc  
      actrl := (ach << 27) | (ach << 11)                        ' setup control value (dual reads)

    1..4 :
      repeat 8
        outa[ADC_CLK] := 0                                      ' lower clock pin
        outa[ADC_DI] := (actrl <-= 1)                           ' output control bit
        awork := (awork << 1) | ina[ADC_DO]                     ' input work bit (reading) 
        outa[ADC_CLK] := 1                                      ' raise the clock line

    5 : 
      outa[ADC_CS] := 1
      analog[ach] := awork & $0FFF
      ach := ++ach & %11   

  if (++astate == 6)
    astate := 0

    
dat { license }

{{

  Copyright (c) 2016 Jon McPhalen  

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}